install.packages("ggplot2")   #installs the plotting package
library(ggplot2)              #loads the plotting package


River_Signatures = data.frame("Sr8786" = c(0.7086, 0.7096, 0.7130, 0.70918), 
                              "River_Reach" = c("Upper Snake", "Lower Snake", "Clearwater", "Ocean")) #creates a dataframe of river signatures

Otolith_Lab_04_analyzed_data$lower_sd = Otolith_Lab_04_analyzed_data$region_mean - Otolith_Lab_04_analyzed_data$region_sd #creates lower standard deviations for each point
Otolith_Lab_04_analyzed_data$upper_sd = Otolith_Lab_04_analyzed_data$region_mean + Otolith_Lab_04_analyzed_data$region_sd #creates lower standard deviations for each point

exclude = c("0") #used to exclude uneeded data 

data_for_plot = Otolith_Lab_04_analyzed_data[!Otolith_Lab_04_analyzed_data$region_name == exclude & 
                                               !Otolith_Lab_04_analyzed_data$lower_sd == exclude &
                                               !Otolith_Lab_04_analyzed_data$lower_sd == exclude,
                                             ] #excludes all of the uneeded data from the dataframe of fish data

Life_Stage_Plot = ggplot(
  data = data_for_plot, 
  aes(x=region_name, y = region_mean, shape = Sample_ID)
  ) #creates a basic plot object with our data in ggplot
  
Life_Stage_Plot + #the following code adds visual pieces to the ggplot
  geom_point() + #adds fish points
  geom_errorbar(
    data = data_for_plot, 
    aes(ymin=lower_sd, ymax = upper_sd), size = .2) +
  geom_hline(data = River_Signatures, 
             aes(yintercept = Sr8786, color = River_Reach), 
             size = 3, alpha = .3) + #adds error bars to points
  labs(x = "Life Stage", y = "87Sr/86Sr", shape = "Sample Name", color = "River Reach") + #adds labels
  facet_wrap(.~Sample_ID, ncol = 1) + #adds faceting, to create a plot for each fish
  theme_bw() #adds a more pleasing theme for the plot



